class PushsController < ApplicationController
  include ActionController::HttpAuthentication::Token
  skip_before_action :verify_authenticity_token

  def token
    param! :app_name, String, required: true
    param! :app_id, String, required: true
    param! :app_secret, String, required: true

    app_setting = Settings.send(params[:app_name].to_sym) rescue {}
    if app_setting['app_id'] == params[:app_id] && app_setting['app_secret'] == params[:app_secret]
      render json: { code: 0, message: 'success', token: generate_token(params[:app_name]), ttl: token_ttl(params[:app_name])}
    else
      render json: { code: -1, message: '应用不存在或者app_id/app_secret不匹配'}
    end
  end
  
  def single
    param! :app_name, String, required: true
    # param! :device_ids_opts, String, required: true
    param! :message, String, required: true
    param! :igetui_opts, String
    param! :app_type, String
    param! :sync_push, String
    param! :user_id, String, required: true
  
    igetui_opts = JSON.parse(params[:igetui_opts]) rescue {}
    message = JSON.parse(params[:message]) rescue {}
    user_id = JSON.parse(params[:user_id]) rescue ""
    
    user_device_exist = UserDevice.where(user_id: user_id).exists?
    unless user_device_exist
      render json: { code: -1, message: 'user_device找不到，user_id错误'}
    end

    token = token_and_options(request).first
    if validate_token(params[:app_name], token)
      push_sync(params[:sync_push], {user_ids: Array.new(user_id), message: message, app_type: params[:app_type], igetui_opts: igetui_opts.merge(pusher_type: 'push_message_to_single')})

      
      render json: { code: 0, message: 'success', describe: '异步任务正在处理'}
    else
      render json: { code: -1, message: 'token 错误/过期'}
    end
    
  end
  
  def batch
    param! :app_name, String, required: true
    # param! :device_ids_opts, String, required: true
    param! :message, String, required: true
    param! :igetui_opts, String
    param! :app_type, String
    param! :sync_push, String
    param! :user_ids, Array, required: true
  
    igetui_opts = JSON.parse(params[:igetui_opts]) rescue {}
    message = JSON.parse(params[:message]) rescue {}
    user_ids = JSON.parse(params[:user_ids]) rescue []

    user_devices_exists = UserDevice.where(user_id: user_ids).exists?
    unless user_devices_exists
      render json: { code: -1, message: 'user_device找不到,user_ids错误'}
    end
  
    token = token_and_options(request).first
    if validate_token(params[:app_name], token)
      push_sync(params[:sync_push], {user_ids: user_ids, message: message, app_type: params[:app_type], igetui_opts: igetui_opts.merge(pusher_type: 'push_message_to_list')})

      render json: { code: 0, message: 'success', describe: '异步任务正在处理'}
    else
      render json: { code: -1, message: 'token 错误/过期'}
    end
    
  end
  
  
  private

  def push_sync(sync_push, opts={})
    if sync_push
      timeout_seconds = $redis.get('push_timeout').to_i
      if timeout_seconds > 0
        Timeout.timeout(timeout_seconds) {
          ::PushWorker.new.perform(opts)
        }
      else
        ::PushWorker.new.perform(opts)
      end
    else
      ::PushWorker.perform_async(opts)
    end
  end
end
