module Igetui
  class Base
    
    def initialize(user_device_ids, opts)
      opts = HashWithIndifferentAccess.new(opts)
      @template_data = opts[:template_data]
      @igetui_opts = opts[:igetui_opts]
      @user_device_ids = user_device_ids
      @pusher = pusher(opts[:device_platform])
      @pusher_type = opts[:pusher_type]
    end
    
    def perform
      @template_data[:push_type] ||= config[:push_type]
      @igetui_opts[:transmission_type] ||= 0
      @igetui_opts[:transmission_content] = @template_data.as_json

      template_type = 'TransmissionTemplate'
      template_data = @igetui_opts
      
      result = push_message_to(@pusher, @pusher_type, template_type, template_data, @user_device_ids)
      ::Log.info("igetui_push: \n template_data: #{template_data}\n user_device_ids: #{@user_device_ids}\n response: #{result}")
    end
    
    private
    def pusher(device_platform)
      case device_platform
        when 'android'
          Igetui::AndroidPusher.pusher
        when 'ios'
          Igetui::IosPusher.pusher
      end
    end

    def push_message_to(pusher, pusher_type, template_type, template_data, client_id)
      client_id_list = Array(client_id)
      template = set_template_data(template_type, template_data)
      
      
      case pusher_type
        when 'push_message_to_single'
          message = IGeTui::SingleMessage.new
          message.data = template
      
          client_id_list.map do |client_id|
            client = IGeTui::Client.new(client_id)
            pusher.push_message_to_single(message, client)
          end
        when 'push_message_to_list'
          message = IGeTui::ListMessage.new
          message.data = template
      
          content_id = pusher.get_content_id(message)
          clients = client_id_list.map { |client_id| IGeTui::Client.new(client_id) }
      
          pusher.push_message_to_list(content_id, clients)
        when 'push_message_to_app'
          message = IGeTui::AppMessage.new
          message.data = template
      
          message.app_id_list = client_id_list
      
          pusher.push_message_to_app(message)
      end
    end

    def set_template_data(template_type, template_data)
      @template ||= "IGeTui::#{template_type}".constantize.new
      case template_type
        when 'NotificationTemplate', 'NotyPopLoadTemplate', 'LinkTemplate'
          template_data.reverse_merge!(Settings.platform_settings.igetui['template_base'])
      
          template_data.each do |k, v|
            @template.send("#{k}=", v)
          end
          @template.set_push_info(*template_data[:push_info]) if template_data.key? :push_info
        when 'TransmissionTemplate'
          @template.transmission_content = template_data[:transmission_content].is_a?(Hash) ? template_data[:transmission_content].to_json : template_data[:transmission_content]
          @template.transmission_type = template_data[:transmission_type] if template_data.key?(:transmission_type)
          @template.set_push_info(*template_data[:push_info]) if template_data.key? :push_info
      end
    end
  end
end
