module Xiaomi
  class Base
    PUSH_URL = 'https://api.xmpush.xiaomi.com/v3/message/regid'.freeze
    BATCH_PUSH_MAX = 80
      
    def initialize(device_ids, message)
      @registration_ids = device_ids
      @payload = message
    end

    def perform
      registration_ids = @registration_ids
      payload = @payload

      registration_ids.each_slice(BATCH_PUSH_MAX) do |registration_ids_array|
        registration_id = registration_ids_array.join(',')
        post(message(payload, registration_id))
      end
    rescue StandardError => err
      ::Log.error(err)
    end

    def message(payload, registration_id)
      message = config
      message[:registration_id] = registration_id
      message[:payload] = payload.to_json
      message
    end

    # notify_type
    # DEFAULT_ALL = -1;
    # DEFAULT_SOUND  = 1;  // 使用默认提示音提示；
    # DEFAULT_VIBRATE = 2;  // 使用默认震动提示；
    # DEFAULT_LIGHTS = 4;   // 使用默认led灯光提示；

    # pass_through
    # 0 表示通知栏消息, 1 表示透传消息
    def config
      {
        restricted_package_name: Settings.platform_settings.xiaomi['package_name'],
        pass_through: 1,
        title: 'xiaomi push',
        description: 'xiaomi push',
        notify_type: 1
      }
    end

    def post(params)
      response = HTTParty.post(PUSH_URL, query: params,
                                        headers: {
                                          'Accept' => 'application/json;',
                                          "Authorization": "key=#{Settings.platform_settings.xiaomi['app_secret']}"
                                        },
                                        timeout: 5)
      ::Log.info("xiaomi_push: \n url: #{PUSH_URL}\n params: #{params}\n response: #{response}")
      response
    end
  end
end
